# Usefull library
library(ggplot2)
library(scales)
library(qpdf)
library(gridExtra)
library(gridtext)
library(dplyr)
library(grid)
library(ggh4x)
library(RColorBrewer)
library(rgdal)




display_type = function (type, bold=FALSE) {

    if (type == "QA") {
        if (bold) {
            disp = bquote(Q[A])
        } else {
            disp = bquote(bold(Q[A]))
        }
    } else if (type == "QMNA") {
        if (bold) {
            disp = bquote(Q[MNA])
        } else {
            disp = bquote(bold(Q[MNA]))
        }
    } else if (type == "VCN10") {
        if (bold) {
            disp = bquote(V[CN10])
        } else {
            disp = bquote(bold(V[CN10]))
        }
    }
    
    return (disp)
}


# Personal theme
theme_ash =
    theme(
        # White background
        panel.background=element_rect(fill='white'),
        # Font
        text=element_text(family='sans'),
        # Border of plot
        panel.border = element_rect(color="grey85",
                                    fill=NA,
                                    size=0.7),
        # Grid
        panel.grid.major.x=element_blank(),
        panel.grid.major.y=element_blank(),
        # Ticks marker
        axis.ticks.x=element_line(color='grey75', size=0.3),
        axis.ticks.y=element_line(color='grey75', size=0.3),
        # Ticks label
        axis.text.x=element_text(color='grey40'),
        axis.text.y=element_text(color='grey40'),
        # Ticks length
        axis.ticks.length=unit(1.5, 'mm'),
        # Ticks minor
        ggh4x.axis.ticks.length.minor=rel(0.5),
        # Title
        plot.title=element_text(size=9, vjust=-2, 
                                hjust=-1E-3, color='grey20'), 
        # Axis title
        axis.title.x=element_blank(),
        axis.title.y=element_blank(),
        # Axis line
        axis.line.x=element_blank(),
        axis.line.y=element_blank(),
        )


time_panel = function (df_data_code, df_trend_code, type, p_threshold=0.1, missRect=FALSE, unit2day=365.25, trend_period=NULL, mean_period=NULL, axis_xlim=NULL, last=FALSE, first=FALSE, color=NULL) {

    # If 'type' is square root apply it to data
    if (type == 'sqrt(Q)') {
        df_data_code$Qm3s = sqrt(df_data_code$Qm3s)
    }
    
    # Compute max of flow
    maxQ = max(df_data_code$Qm3s, na.rm=TRUE)
    # Get the magnitude of the max of flow
    power = get_power(maxQ) 

    # Normalize the max flow by it's magnitude
    maxQtmp = maxQ/10^power
    # Compute the spacing between y ticks
    if (maxQtmp >= 5) {
        dbrk = 1.0
    } else if (maxQtmp < 5 & maxQtmp >= 3) {
        dbrk = 0.5
    } else if (maxQtmp < 3 & maxQtmp >= 2) {
        dbrk = 0.4
    } else if (maxQtmp < 2 & maxQtmp >= 1) {
        dbrk = 0.2
    } else if (maxQtmp < 1) {
        dbrk = 0.1
    }
    
    # Get the spacing in the right magnitude
    dbrk = dbrk * 10^power
    # Fix the accuracy for label
    accuracy = NULL
    
    # Time span in the unit of time
    dDate = as.numeric(df_data_code$Date[length(df_data_code$Date)] -
                       df_data_code$Date[1]) / unit2day
    # Compute the spacing between x ticks
    if (dDate >= 100) {
        datebreak = 25
        dateminbreak = 5
    } else if (dDate < 100 & dDate >= 50) {
        datebreak = 10
        dateminbreak = 1
    } else if (dDate < 50) {
        datebreak = 5
        dateminbreak = 1
    }
    
    # Open new plot
    p = ggplot() + theme_ash 
        
    # If it is the lats plot of the pages or not
    if (last) {
        if (first) {
            p = p +
                theme(plot.margin=margin(5, 5, 5, 5, unit="mm"))
        } else {
            p = p +
                theme(plot.margin=margin(0, 5, 5, 5, unit="mm"))
        }
        
    # If it is the first plot of the pages or not
    } else {
        if (first) {
            p = p +
                theme(plot.margin=margin(5, 5, 0, 5, unit="mm"))
        } else {
            p = p +
                theme(plot.margin=margin(0, 5, 0, 5, unit="mm"))
        }
    } 

    ## Sub period background ##
    if (!is.null(trend_period)) {
        
        # trend_period = as.list(trend_period)
        # Imin = 10^99
        # for (per in trend_period) {
        #     I = interval(per[1], per[2])
        #     if (I < Imin) {
        #         Imin = I
        #         trend_period_min = as.Date(per)
        #     }
        # }
        # p = p + 
        #     geom_rect(aes(xmin=min(df_data_code$Date),
        #                   ymin=0, 
        #                   xmax=trend_period_min[1],
        #                   ymax= maxQ*1.1),
        #               linetype=0, fill='grey97') +
            
        #     geom_rect(aes(xmin=trend_period_min[2],
        #                   ymin=0, 
        #                   xmax=max(df_data_code$Date), 
        #                   ymax= maxQ*1.1),
        #               linetype=0, fill='grey97') 

        # Convert trend period to list if it is not
        trend_period = as.list(trend_period)
        # Fix a disproportionate minimum for period
        Imin = 10^99
        # For all the sub period of analysis in 'trend_period'
        for (per in trend_period) {
            # Compute time interval of period
            I = interval(per[1], per[2])
            # If it is the smallest interval
            if (I < Imin) {
                # Store it
                Imin = I
                # Fix min period of analysis
                trend_period_min = as.Date(per)
            }
        }
        # Search for the index of the closest existing date 
        # to the start of the min period of analysis
        idMinPer = which.min(abs(df_data_code$Date - trend_period_min[1]))
        # Same for the end of the min period of analysis
        idMaxPer = which.min(abs(df_data_code$Date - trend_period_min[2]))
        # Get the start and end date associated
        minPer = df_data_code$Date[idMinPer]
        maxPer = df_data_code$Date[idMaxPer]

        # If it is not a flow or sqrt of flow time serie
        if (type != 'sqrt(Q)' & type != 'Q') {
            # If there is an 'axis_lim'
            if (!is.null(axis_xlim)) {
                # If the temporary start of period is smaller 
                # than the fix start of x axis limit
                if (minPer < axis_xlim[1]) {
                    # Set the start of the period to the start of
                    # the x axis limit
                    minPer = axis_xlim[1]
                }
            }
        }
        # If it is not a flow or sqrt of flow time serie
        if (type != 'sqrt(Q)' & type != 'Q') {
            # If there is an 'axis_lim'
            if (!is.null(axis_xlim)) {
                # If the temporary end of period plus one year 
                # is smaller than the fix end of x axis limit
                if (maxPer + years(1) < axis_xlim[2]) {
                    # Add one year the the temporary end of period
                    maxPer = maxPer + years(1)
                } else {
                    # Set the start of the period to the start of
                    # the x axis limit
                    maxPer = axis_xlim[2]
                }
                # Add one year the the temporary end of period
                # if there is no 'axis_lim'
            } else {
                    maxPer = maxPer + years(1)
            }
        }

        # Draw rectangle to delimiting the sub period
        p = p + 
            geom_rect(aes(xmin=minPer,
                          ymin=0, 
                          xmax=maxPer,
                          ymax= maxQ*1.1),
                      linetype=0, fill='grey97')
    }

    ## Mean step ##
    # If there is a 'mean_period'
    if (!is.null(mean_period)) {
        # Convert 'mean_period' to list
        mean_period = as.list(mean_period)
        # Number of mean period
        nPeriod_mean = length(mean_period)

        # Blank tibble to store variable in order to plot
        # rectangle for mean period
        plot_mean = tibble()
        # Blank tibble to store variable in order to plot
        # upper limit of rectangle for mean period
        plot_line = tibble()
        # For all mean period
        for (j in 1:nPeriod_mean) {
            # Get the current start and end of the sub period
            Start_mean = mean_period[[j]][1]
            End_mean = mean_period[[j]][2]

            # Extract the data corresponding to this sub period
            df_data_code_per =
                df_data_code[df_data_code$Date >= Start_mean 
                             & df_data_code$Date <= End_mean,]

            # Min for the sub period
            xmin = min(df_data_code_per$Date)
            # If the min over the sub period is greater
            # than the min of the entier period and
            # it is not the first sub period
            if (xmin > min(df_data_code$Date) & j != 1) {
                # Substract 6 months to be in the middle of
                # the previous year
                xmin = xmin - months(6)
            }
            # If it is not a flow or sqrt of flow time serie and
            # it is the first period
            if (type != 'sqrt(Q)' & type != 'Q' & j == 1) {
                # If there is an x axis limit
                if (!is.null(axis_xlim)) {
                    # If the min of the period is before the x axis min
                    if (xmin < axis_xlim[1]) {
                        # The min for the sub period is the x axis
                        xmin = axis_xlim[1]
                    }
                }
            }
            # Max for the sub period
            xmax = max(df_data_code_per$Date)
            # If the max over the sub period is smaller
            # than the max of the entier period and
            # it is not the last sub period
            if (xmax < max(df_data_code$Date) & j != nPeriod_mean) {
                # Add 6 months to be in the middle of
                # the following year
                xmax = xmax + months(6)
            }
            # If it is not a flow or sqrt of flow time serie and
            # it is the last period
            if (type != 'sqrt(Q)' & type != 'Q' & j == nPeriod_mean) {
                # If there is an x axis limit
                if (!is.null(axis_xlim)) {
                    # If the max of the period plus 1 year
                    # is smaller thant the max of the x axis limit
                    if (xmax + years(1) < axis_xlim[2]) {
                        # Add one year to the max to include
                        # the entire last year graphically
                        xmax = xmax + years(1)
                    } else {
                        # The max of this sub period is the max
                        # of the x axis limit
                        xmax = axis_xlim[2]
                    }
                    # If there is no axis limit
                } else {
                    # Add one year to the max to include
                    # the entire last year graphically
                    xmax = xmax + years(1)
                }
            }
            # Mean of the flow over the sub period
            ymax = mean(df_data_code_per$Qm3s, na.rm=TRUE)

            # Create temporary tibble with variable
            # to create rectangle for mean step
            plot_meantmp = tibble(xmin=xmin, xmax=xmax, 
                                  ymin=0, ymax=ymax, period=j)
            # Bind it to the main tibble to store it with other period
            plot_mean = bind_rows(plot_mean, plot_meantmp)

            # Create vector for the upper limit of the rectangle
            abs = c(xmin, xmax)
            ord = c(ymax, ymax)
            
            # Create temporary tibble with variable
            # to create upper limit for rectangle
            plot_linetmp = tibble(abs=abs, ord=ord, period=j)
            # Bind it to the main tibble to store it with other period
            plot_line =  bind_rows(plot_line, plot_linetmp)
        }
        # Plot rectangles
        p = p + 
            geom_rect(data=plot_mean,
                      aes(xmin=xmin, ymin=ymin, 
                          xmax=xmax, ymax=ymax),
                      linetype=0, fill='grey93')
        # Plot upper line for rectangle
        p = p +
            geom_line(data=plot_line,
                      aes(x=abs, y=ord, group=period),
                      color='grey85',
                      size=0.15)
        
        # for all the sub periods except the last one
        for (i in 1:(nPeriod_mean-1)) {
            # The y limit of rectangle is the max of
            # the two neighboring mean step rectangle
            yLim = max(c(plot_mean$ymax[i], plot_mean$ymax[i+1]))
            # The x limit is the x max of the ith rectangle
            xLim = plot_mean$xmax[i]
            # Make a tibble to store data
            plot_lim = tibble(x=c(xLim, xLim), y=c(0, yLim))
            # Plot the limit of rectangles
            p = p + 
                geom_line(data=plot_lim, aes(x=x, y=y),
                          linetype='dashed', size=0.15, color='grey85') 
        }

    }

    ### Grid ###
    # If there is no axis limit
    if (is.null(axis_xlim)) {
        # The min and the max is set by
        # the min and the max of the date data 
        xmin = min(df_data_code$Date)
        xmax = max(df_data_code$Date)
    } else {
        # Min and max is set with the limit axis parameter
        xmin = axis_xlim[1]
        xmax = axis_xlim[2]
    }
    # Create a vector for all the y grid position
    ygrid = seq(0, maxQ*10, dbrk)
    # Blank vector to store position
    ord = c() 
    abs = c()
    # For all the grid element
    for (i in 1:length(ygrid)) {
        # Store grid position
        ord = c(ord, rep(ygrid[i], times=2))
        abs = c(abs, xmin, xmax)
    }
    # Create a tibble to store all the position
    plot_grid = tibble(abs=as.Date(abs), ord=ord)
    # Plot the y grid
    p = p +
        geom_line(data=plot_grid, 
                  aes(x=abs, y=ord, group=ord),
                  color='grey85',
                  size=0.15)

    ### Data ###
    # If it is a square root flow or flow
    if (type == 'sqrt(Q)' | type == 'Q') {
        # Plot the data as line
        p = p +
            geom_line(aes(x=df_data_code$Date, y=df_data_code$Qm3s),
                      color='grey20',
                      size=0.3,
                      lineend="round")
    } else {
        # Plot the data as point
        p = p +
            geom_point(aes(x=df_data_code$Date, y=df_data_code$Qm3s),
                       shape=21, color='grey50', fill='grey97', size=1)
    }

    ### Missing data ###
    # If the option is TRUE
    if (missRect) {
        # Remove NA data
        NAdate = df_data_code$Date[is.na(df_data_code$Qm3s)]
        # Get the difference between each point of date data without NA
        dNAdate = diff(NAdate)
        # If difference of day is not 1 then
        # it is TRUE for the beginning of each missing data period 
        NAdate_Down = NAdate[append(Inf, dNAdate) != 1]
        # If difference of day is not 1 then
        # it is TRUE for the ending of each missing data period 
        NAdate_Up = NAdate[append(dNAdate, Inf) != 1]

        # Plot the missing data period
        p = p +
            geom_rect(aes(xmin=NAdate_Down, 
                          ymin=0, 
                          xmax=NAdate_Up, 
                          ymax=maxQ*1.1),
                      linetype=0, fill='Wheat', alpha=0.4)
    }

    ### Trend ###
    # If there is trends
    if (!is.null(df_trend_code)) {

        # Extract starting period of trends
        Start = df_trend_code$period_start
        # Get the name of the different period
        UStart = levels(factor(Start))
        # Same for ending
        End = df_trend_code$period_end
        UEnd = levels(factor(End))

        # Compute the max of different start and end
        # so the number of different period
        nPeriod_trend = max(length(UStart), length(UEnd))

        # Blank tibble to store trend data and legend data
        plot_trend = tibble()
        leg_trend = tibble()
        # For all the different period
        for (i in 1:nPeriod_trend) {

            # Get the trend associated to the first period
            df_trend_code_per = 
                df_trend_code[df_trend_code$period_start == Start[i] 
                              & df_trend_code$period_end == End[i],]
            
            # Number of trend selected
            Ntrend = nrow(df_trend_code_per)
            # If the number of trend is greater than a unique one
            if (Ntrend > 1) {
                # Extract only the first hence it is the same period
                df_trend_code_per = df_trend_code_per[1,]
            }            

            # Search for the index of the closest existing date 
            # to the start of the trend period of analysis
            iStart = which.min(abs(df_data_code$Date - Start[i]))
            # Same for the end
            iEnd = which.min(abs(df_data_code$Date - End[i]))

            # Get the start and end date associated
            xmin = df_data_code$Date[iStart]
            xmax = df_data_code$Date[iEnd]

            # If there is a x axis limit
            if (!is.null(axis_xlim)) {
                # If the min of the current period
                # is smaller than the min of the x axis limit
                if (xmin < axis_xlim[1]) {
                    # The min of the period is the min
                    # of the x axis limit
                    xmin = axis_xlim[1]
                }
                # Same for end
                if (xmax > axis_xlim[2]) {
                    xmax = axis_xlim[2]
                } 
            }

            # Create vector to store x data
            abs = c(xmin, xmax)
            # Convert the number of day to the unit of the period
            abs_num = as.numeric(abs) / unit2day
            # Compute the y of the trend
            ord = abs_num * df_trend_code_per$trend +
                df_trend_code_per$intercept

            # Create temporary tibble with variable to plot trend
            # for each period
            plot_trendtmp = tibble(abs=abs, ord=ord, period=i)
            # Bind it to the main tibble to store it with other period
            plot_trend = bind_rows(plot_trend, plot_trendtmp)

            # If there is a x axis limit
            if (!is.null(axis_xlim)) {
                # The x axis limit is selected
                codeDate = axis_xlim
            } else {
                # The entire date data is selected
                codeDate = df_data_code$Date
            }
            # The flow data is extract
            codeQ = df_data_code$Qm3s

            # Position of the x beginning and end of the legend symbol
            x = gpct(2, codeDate, shift=TRUE)
            xend = x + gpct(3, codeDate)

            # Position of the y beginning and end of the legend symbol
            dy = gpct(7, codeQ, ref=0)
            y = gpct(100, codeQ, ref=0) - (i-1)*dy
            yend = y

            # Position of x for the beginning of the associated text
            xt = xend + gpct(1, codeDate)

            # Position of the background rectangle of the legend
            xminR = x - gpct(1, codeDate)
            yminR = y - gpct(4, codeQ, ref=0)
            xmaxR = x + gpct(24, codeDate)
            ymaxR = y + gpct(5, codeQ, ref=0)

            # Get the tendance analyse
            trend = df_trend_code_per$trend
            # Compute the magnitude of the trend
            power = get_power(trend)
            # Convert it to character
            powerC = as.character(power)
            # Get the power of ten of magnitude
            brk = 10^power
            # Convert trend to character for sientific expression
            trendC = as.character(round(trend / brk, 2))

            # Create temporary tibble with variable to plot legend
            leg_trendtmp = tibble(x=x, xend=xend, 
                                  y=y, yend=yend, 
                                  xt=xt,
                                  trendC=trendC,
                                  powerC=powerC,
                                  xminR=xminR, yminR=yminR,
                                  xmaxR=xmaxR, ymaxR=ymaxR,
                                  period=i)
            # Bind it to the main tibble to store it with other period
            leg_trend = bind_rows(leg_trend, leg_trendtmp)  
        }

        # For all periods
        for (i in 1:nPeriod_trend) {
            # Extract the trend of the current sub period
            leg_trend_per = leg_trend[leg_trend$period == i,]

            # Plot the background for legend
            p = p +
                geom_rect(data=leg_trend_per,
                          aes(xmin=xminR, 
                              ymin=yminR, 
                              xmax=xmaxR, 
                              ymax=ymaxR),
                          linetype=0, fill='white', alpha=0.5)
        }
        
        # For all periods
        for (i in 1:nPeriod_trend) {
            # Extract the trend of the current sub period
            leg_trend_per = leg_trend[leg_trend$period == i,]

            # Get the character variable for naming the trend
            trendC = leg_trend_per$trendC
            powerC = leg_trend_per$powerC

            # Create the name of the trend
            label = bquote(bold(.(trendC)~'x'~'10'^{.(powerC)})~'['*m^{3}*'.'*s^{-1}*'.'*an^{-1}*']')

            # Plot the trend symbole and value of the legend
            p = p +
                annotate("segment",
                         x=leg_trend_per$x, xend=leg_trend_per$xend,
                         y=leg_trend_per$y, yend=leg_trend_per$yend,
                         color=color[i],
                         linetype='solid',
                         lwd=1) +

                annotate("text",
                         label=label, size=3,
                         x=leg_trend_per$xt, y=leg_trend_per$y, 
                         hjust=0, vjust=0.4,
                         color=color[i])
        }

        # For all periods
        for (i in 1:nPeriod_trend) {
            # Extract the trend of the current sub period
            plot_trend_per = plot_trend[plot_trend$period == i,]
            
            # Plot the line of white background of each trend
            p = p + 
                geom_line(data=plot_trend_per, 
                          aes(x=abs, y=ord),
                          color='white',
                          linetype='solid',
                          size=1,
                          lineend="round")
        }

        # For all periods
        for (i in 1:nPeriod_trend) {
            # Extract the trend of the current sub period
            plot_trend_per = plot_trend[plot_trend$period == i,]

            # Plot the line of trend
            p = p + 
                geom_line(data=plot_trend_per, 
                          aes(x=abs, y=ord),
                          color=color[i],
                          linetype='solid',
                          size=0.5,
                          lineend="round")
        }
    }

    # Title
    p = p +
        ggtitle(bquote(bold(.(type))~~'['*m^{3}*'.'*s^{-1}*']'))

    # If the is no x axis limit
    if (is.null(axis_xlim)) {
        # Parameters of the x axis contain the limit of the date data
        p = p +
            scale_x_date(date_breaks=paste(
                             as.character(datebreak), 
                             'year', sep=' '),
                         date_minor_breaks=paste(
                             as.character(dateminbreak), 
                             'year', sep=' '),
                         guide='axis_minor',
                         date_labels="%Y",
                         limits=c(min(df_data_code$Date), 
                                  max(df_data_code$Date)),
                         expand=c(0, 0))
    } else {
        # Parameters of the x axis contain the x axis limit
        p = p +
            scale_x_date(date_breaks=paste(
                             as.character(datebreak), 
                             'year', sep=' '),
                         date_minor_breaks=paste(
                             as.character(dateminbreak), 
                             'year', sep=' '),
                         guide='axis_minor',
                         date_labels="%Y",
                         limits=axis_xlim,
                         expand=c(0, 0))
    }

    # Parameters of the y axis
    p = p +
        scale_y_continuous(breaks=seq(0, maxQ*10, dbrk),
                           limits=c(0, maxQ*1.1),
                           expand=c(0, 0),
                           labels=label_number(accuracy=accuracy))

    return(p)
}


matrice_panel = function (list_df2plot, df_meta, trend_period, slice=NULL, outdirTmp='', outnameTmp='matrix') {

    nbp = length(list_df2plot)
    
    # Get all different stations code
    Code = levels(factor(df_meta$code))
    nCode = length(Code)

    if (!is.null(slice)) {

        # By
        nMat = as.integer(nCode/slice) + 1
        sublist_df2plot = list_df2plot

        for (i in 1:nMat) {

            subdf_meta = df_meta[(slice*(i-1)+1):(slice*i),]
            subdf_meta = subdf_meta[!is.na(subdf_meta$code),]
            subCode = subdf_meta$code
            

            for (j in 1:nbp) {
                df_datatmp = list_df2plot[[j]]$data
                df_trendtmp = list_df2plot[[j]]$trend

                subdf_data = df_datatmp[(df_datatmp$code %in% subCode),]
                subdf_trend = df_trendtmp[(df_trendtmp$code %in% subCode),]
                
                sublist_df2plot[[j]]$data = subdf_data
                sublist_df2plot[[j]]$trend = subdf_trend
            }
            
            mat = matrice_panel(sublist_df2plot, 
                                subdf_meta,
                                trend_period=trend_period,
                                outdirTmp=outdirTmp,
                                outnameTmp=paste(outnameTmp,
                                                 '_', i,
                                                 sep=''))
        }
        
    } else { 

        print(paste('matrix :', outnameTmp))
        # nbp = length(list_df2plot)

        # # Get all different stations code
        # Code = levels(factor(df_meta$code))
        # nCode = length(Code)

        df_trend = list_df2plot[[1]]$trend

        nPeriod_max = 0
        for (code in Code) {
            
            df_trend_code = df_trend[df_trend$code == code,]

            Start = df_trend_code$period_start
            UStart = levels(factor(Start))
            
            End = df_trend_code$period_end
            UEnd = levels(factor(End))
            
            nPeriod = max(length(UStart), length(UEnd))

            if (nPeriod > nPeriod_max) {
                nPeriod_max = nPeriod
            }
        }
        
        Start_code = vector(mode='list', length=nCode)
        End_code = vector(mode='list', length=nCode)
        Code_code = vector(mode='list', length=nCode)
        Periods_code = vector(mode='list', length=nCode)

        for (j in 1:nCode) {
            
            code = Code[j]

            df_trend_code = df_trend[df_trend$code == code,]

            Start = df_trend_code$period_start
            UStart = levels(factor(Start))
            
            End = df_trend_code$period_end
            UEnd = levels(factor(End))
            
            nPeriod = max(length(UStart), length(UEnd))

            Periods = c()

            for (i in 1:nPeriod_max) {
                Periods = append(Periods, 
                                 paste(substr(Start[i], 1, 4),
                                       substr(End[i], 1, 4),
                                       sep=' / '))
            }

            Start_code[[j]] = Start
            End_code[[j]] = End
            Code_code[[j]] = code
            Periods_code[[j]] = Periods
            
        }

        TrendMean_code = array(rep(1, nPeriod_max*nbp*nCode),
                               dim=c(nPeriod_max, nbp, nCode))

        for (j in 1:nPeriod_max) {

            for (k in 1:nCode) {
                
                code = Code[k]
                
                for (i in 1:nbp) {
                    
                    df_data = list_df2plot[[i]]$data
                    df_trend = list_df2plot[[i]]$trend
                    p_threshold = list_df2plot[[i]]$p_threshold

                    df_data_code = df_data[df_data$code == code,] 
                    df_trend_code = df_trend[df_trend$code == code,]

                    Start = Start_code[Code_code == code][[1]][j]
                    End = End_code[Code_code == code][[1]][j]
                    Periods = Periods_code[Code_code == code][[1]][j]

                    df_data_code_per =
                        df_data_code[df_data_code$Date >= Start 
                                     & df_data_code$Date <= End,]

                    df_trend_code_per = 
                        df_trend_code[df_trend_code$period_start == Start 
                                      & df_trend_code$period_end == End,]

                    Ntrend = nrow(df_trend_code_per)
                    if (Ntrend > 1) {
                        df_trend_code_per = df_trend_code_per[1,]
                    }
                    
                    dataMean = mean(df_data_code_per$Qm3s, na.rm=TRUE)
                    trendMean = df_trend_code_per$trend / dataMean

                    TrendMean_code[j, i, k] = trendMean
                }
            }
        }

        minTrendMean = apply(TrendMean_code, c(1, 2), min, na.rm=TRUE)
        maxTrendMean = apply(TrendMean_code, c(1, 2), max, na.rm=TRUE)

        Periods_mat = c()
        NPeriod_mat = c()
        Type_mat = list()
        Code_mat = c()
        Pthresold_mat = c()
        TrendMean_mat = c()
        DataMean_mat = c()
        Fill_mat = c()
        Color_mat = c()

        for (j in 1:nPeriod_max) {

            for (code in Code) {
                
                for (i in 1:nbp) {
                    df_data = list_df2plot[[i]]$data
                    df_trend = list_df2plot[[i]]$trend
                    p_threshold = list_df2plot[[i]]$p_threshold
                    type = list_df2plot[[i]]$type
                    
                    df_data_code = df_data[df_data$code == code,] 
                    df_trend_code = df_trend[df_trend$code == code,]

                    Start = Start_code[Code_code == code][[1]][j]
                    End = End_code[Code_code == code][[1]][j]
                    Periods = Periods_code[Code_code == code][[1]][j]

                    df_data_code_per =
                        df_data_code[df_data_code$Date >= Start 
                                     & df_data_code$Date <= End,]

                    df_trend_code_per = 
                        df_trend_code[df_trend_code$period_start == Start 
                                      & df_trend_code$period_end == End,]

                    Ntrend = nrow(df_trend_code_per)
                    if (Ntrend > 1) {
                        df_trend_code_per = df_trend_code_per[1,]
                    }
                    
                    dataMean = mean(df_data_code_per$Qm3s, na.rm=TRUE)
                    trendMean = df_trend_code_per$trend / dataMean

                    if (df_trend_code_per$p <= p_threshold){
                        color_res = get_color(trendMean, 
                                              minTrendMean[j, i],
                                              maxTrendMean[j, i],
                                              palette_name='perso',
                                              reverse=TRUE)

                        fill = color_res$color
                        color = 'white'
                        Pthresold = p_thresold
                    } else { 
                        fill = 'white'
                        color = 'grey85'  
                        Pthresold = NA
                    }

                    Periods_mat = append(Periods_mat, Periods)
                    NPeriod_mat = append(NPeriod_mat, j)
                    Type_mat = append(Type_mat, type)
                    Code_mat = append(Code_mat, code)
                    Pthresold_mat = append(Pthresold_mat, Pthresold)
                    TrendMean_mat = append(TrendMean_mat, trendMean)
                    DataMean_mat = append(DataMean_mat, dataMean)
                    Fill_mat = append(Fill_mat, fill)
                    Color_mat = append(Color_mat, color)
                }
            }
        }

        height = length(Code)
        width = nbp * 2 * nPeriod_max + nPeriod_max

        options(repr.plot.width=width, repr.plot.height=height)
        
        mat = ggplot() + theme_ash + 

    theme(
        panel.border=element_blank(),
        axis.text.x=element_blank(),
        axis.text.y=element_blank(),
        axis.ticks.y=element_blank(),
        axis.ticks.x=element_blank(),
        axis.title.y=element_blank(),
        )
        
        # xt = -1
        # yt = height + 1.75
        # Title = bquote(bold(Territoire))
        
        # mat = mat +
        #     annotate("text", x=xt, y=yt,
        #              label=Title,
        #              hjust=0, vjust=0.5, 
        #              size=6, color="#00A3A8")
        
        for (j in 1:nPeriod_max) {

            Type_mat_per = Type_mat[NPeriod_mat == j]
            Code_mat_per = Code_mat[NPeriod_mat == j]
            Pthresold_mat_per = Pthresold_mat[NPeriod_mat == j]
            TrendMean_mat_per = TrendMean_mat[NPeriod_mat == j]
            DataMean_mat_per = DataMean_mat[NPeriod_mat == j]
            Fill_mat_per = Fill_mat[NPeriod_mat == j]
            Color_mat_per = Color_mat[NPeriod_mat == j]

            Xtmp = as.integer(factor(as.character(Type_mat_per)))

            Xc = j + (j - 1)*nbp*2        
            Xm = Xtmp + (j - 1)*nbp*2 + j
            X = Xtmp + (j - 1)*nbp*2 + nbp + j

            Y = as.integer(factor(Code_mat_per))

            x = Xc - 0.4
            xend = X[length(X)] + 0.25
            y = height + 1
            yend = height + 1

            mat = mat +
                annotate("segment",
                         x=x, xend=xend,
                         y=y, yend=yend, 
                         color="grey40", size=0.35)
            
            yt = y + 0.15
            Start = trend_period[[j]][1]
            End = trend_period[[j]][2]
            periodName = bquote(bold('Priode')~bold(.(as.character(j))))
            # bquote(bold(.(Start))~'/'~bold(.(End)))

            mat = mat +
                annotate("text", x=x, y=yt,
                         label=periodName,
                         hjust=0, vjust=0.5, 
                         size=3, color='grey40')
            
            for (i in 1:length(X)) {
                mat = mat +

    gg_circle(r=0.45, xc=X[i], yc=Y[i],
              fill=Fill_mat_per[i], color=Color_mat_per[i]) +

    gg_circle(r=0.45, xc=Xm[i], yc=Y[i],
              fill='white', color='grey40') +

    gg_circle(r=0.45, xc=Xc, yc=Y[i],
              fill='white', color='grey40') 
            }

            for (i in 1:length(TrendMean_mat_per)) {
                trendMean = TrendMean_mat_per[i]
                trendC = round(trendMean*100, 2)

                if (!is.na(Pthresold_mat_per[i])) {
                    Tcolor = 'white'
                } else {
                    Tcolor = 'grey85'
                }
                
                dataMean = round(DataMean_mat_per[i], 2)

                mat = mat +
                    annotate('text', x=X[i], y=Y[i],
                             label=trendC,
                             hjust=0.5, vjust=0.5, 
                             size=3, color=Tcolor) + 

    annotate('text', x=Xm[i], y=Y[i],
             label=dataMean,
             hjust=0.5, vjust=0.5, 
             size=3, color='grey40')
            }

            mat = mat +
                annotate('text', x=Xc, y=max(Y) + 0.85,
                         label=bquote(bold('Dbut')),
                         hjust=0.5, vjust=0.5, 
                         size=3, color='grey20') + 

    annotate('text', x=Xc, y=max(Y) + 0.6,
             label=bquote(bold('Fin')),
             hjust=0.5, vjust=0.5, 
             size=3, color='grey20')

            for (i in 1:nbp) {
                type = list_df2plot[[i]]$type
                mat = mat +
                    annotate('text', x=X[i], y=max(Y) + 0.7,
                             label=bquote(.(type)),
                             hjust=0.5, vjust=0.5, 
                             size=3.5, color='grey20') +
                    
                    annotate('text', x=Xm[i], y=max(Y) + 0.7,
                             label=bquote(''*.(type)),
                             hjust=0.5, vjust=0.5, 
                             size=3.5, color='grey20')  
            }
        }
        
        for (i in 1:length(Code)) {

            code = Code[i]
            name = df_meta[df_meta$code == code,]$nom
            ncharMax = 30
            if (nchar(name) > ncharMax) {
                name = paste(substr(name, 1, ncharMax), '...', sep='')
            }

            mat = mat +
                annotate('text', x=0.3, y=i + 0.14,
                         label=bquote(bold(.(code))),
                         hjust=1, vjust=0.5, 
                         size=3.5, color="#00A3A8") +
                
                annotate('text', x=0.3, y=i - 0.14,
                         label=name,
                         hjust=1, vjust=0.5, 
                         size=3.5, color="#00A3A8")


            for (j in 1:nPeriod_max) {
                Xc = j + (j - 1)*nbp*2   
                label = Periods_code[Code_code == code][[1]][j]
                periodStart = substr(label, 1, 4)
                periodEnd = substr(label, 8, 11)
                
                mat = mat +
                    annotate('text', x=Xc, y=i + 0.13,
                             label=bquote(bold(.(periodStart))),
                             hjust=0.5, vjust=0.5, 
                             size=3, color='grey40') + 
                    
                    annotate('text', x=Xc, y=i - 0.13,
                             label=bquote(bold(.(periodEnd))),
                             hjust=0.5, vjust=0.5, 
                             size=3, color='grey40')
            }
        }

        mat = mat +

        coord_fixed() +
    
        scale_x_continuous(limits=c(1 - rel(6), 
                                    width + rel(0.5)),
                           expand=c(0, 0)) + 
    
        scale_y_continuous(limits=c(1 - rel(0.5), 
                                    height + rel(1.5)),
                           expand=c(0, 0))
        
        # Saving matrix plot
        ggsave(plot=mat, 
               path=outdirTmp,
               filename=paste(outnameTmp, '.pdf', sep=''),
               width=29.7, height=21, units='cm', dpi=100)
    }
}



map_panel = function (list_df2plot, df_meta, idPer, computer_data_path, fr_shpdir, fr_shpname, bs_shpdir=bs_shpdir, bs_shpname=bs_shpname, rv_shpdir, rv_shpname, outdirTmp) {
    
    fr_shppath = file.path(computer_data_path, fr_shpdir, fr_shpname)
    rv_shppath = file.path(computer_data_path, rv_shpdir, rv_shpname)
    bs_shppath = file.path(computer_data_path, bs_shpdir, bs_shpname)

    # France
    fr_spdf = readOGR(dsn=fr_shppath, verbose=FALSE)    
    proj4string(fr_spdf) = CRS("+proj=longlat +ellps=WGS84")
    # Trasnformation en Lambert93
    france = spTransform(fr_spdf, CRS("+init=epsg:2154"))
    df_france = tibble(fortify(france))

    # Bassin hydrographique
    bassin = readOGR(dsn=bs_shppath, verbose=FALSE)
    df_bassin = tibble(fortify(bassin))
    
    # Rseau hydrographique
    # river = readOGR(dsn=rv_shppath, verbose=FALSE) ### trop long ###
    # river = river[which(river$Classe == 1),]
    # df_river = tibble(fortify(river))

    nbp = length(list_df2plot)
  
    # Get all different stations code
    Code = levels(factor(df_meta$code))
    nCode = length(Code)
    
    df_trend = list_df2plot[[1]]$trend
    
    nPeriod_max = 0
    for (code in Code) {
        
        df_trend_code = df_trend[df_trend$code == code,]
        
        Start = df_trend_code$period_start
        UStart = levels(factor(Start))
        
        End = df_trend_code$period_end
        UEnd = levels(factor(End))
        
        nPeriod = max(length(UStart), length(UEnd))
        
        if (nPeriod > nPeriod_max) {
            nPeriod_max = nPeriod
        }
    }
    
    Start_code = vector(mode='list', length=nCode)
    End_code = vector(mode='list', length=nCode)
    Code_code = vector(mode='list', length=nCode)
    Periods_code = vector(mode='list', length=nCode)

    for (j in 1:nCode) {
        
        code = Code[j]

        df_trend_code = df_trend[df_trend$code == code,]

        Start = df_trend_code$period_start
        UStart = levels(factor(Start))
        
        End = df_trend_code$period_end
        UEnd = levels(factor(End))
        
        nPeriod = max(length(UStart), length(UEnd))

        Periods = c()

        for (i in 1:nPeriod_max) {
            Periods = append(Periods, 
                             paste(substr(Start[i], 1, 4),
                                   substr(End[i], 1, 4),
                                   sep=' / '))
        }

        Start_code[[j]] = Start
        End_code[[j]] = End
        Code_code[[j]] = code
        Periods_code[[j]] = Periods
        
    }

    TrendMean_code = array(rep(1, nPeriod_max*nbp*nCode),
                           dim=c(nPeriod_max, nbp, nCode))

    for (j in 1:nPeriod_max) {

        for (k in 1:nCode) {
            
            code = Code[k]
            
            for (i in 1:nbp) {
                
                df_data = list_df2plot[[i]]$data
                df_trend = list_df2plot[[i]]$trend
                p_threshold = list_df2plot[[i]]$p_threshold

                df_data_code = df_data[df_data$code == code,] 
                df_trend_code = df_trend[df_trend$code == code,]

                Start = Start_code[Code_code == code][[1]][j]
                End = End_code[Code_code == code][[1]][j]
                Periods = Periods_code[Code_code == code][[1]][j]

                df_data_code_per =
                    df_data_code[df_data_code$Date >= Start 
                                 & df_data_code$Date <= End,]

                df_trend_code_per = 
                    df_trend_code[df_trend_code$period_start == Start 
                                  & df_trend_code$period_end == End,]

                Ntrend = nrow(df_trend_code_per)
                if (Ntrend > 1) {
                    df_trend_code_per = df_trend_code_per[1,]
                }
                
                dataMean = mean(df_data_code_per$Qm3s, na.rm=TRUE)
                trendMean = df_trend_code_per$trend / dataMean

                TrendMean_code[j, i, k] = trendMean
            }
        }
    }

    minTrendMean = apply(TrendMean_code, c(1, 2), min, na.rm=TRUE)
    maxTrendMean = apply(TrendMean_code, c(1, 2), max, na.rm=TRUE)    

    ncolor = 256
    nbTick = 10
    
    for (i in 1:nbp) {
        
        outname = paste('map_', i, sep='')
        print(paste('map :', outname))

        type = list_df2plot[[i]]$type
        
        map = ggplot() + theme_ash +
            
            theme(panel.border = element_blank()) +
        
            coord_fixed() +
            
            geom_polygon(data=df_france,
                         aes(x=long, y=lat, group=group),
                         color=NA, fill="grey97") +
            
            # geom_path(data=df_river,
            # aes(x=lon, y=lat, group=group),
            # color="grey85", size=0.15) +
            
            geom_polygon(data=df_bassin,
                         aes(x=long, y=lat, group=group),
                         color="grey70", fill=NA, size=0.1) +
            
            geom_polygon(data=df_france,
                         aes(x=long, y=lat, group=group),
                         color="grey40", fill=NA, size=0.2)

        map = map +
                        
            coord_sf(xlim=c(280000, 790000),
                     ylim=c(6110000, 6600000),
                     expand=FALSE)

        map = map +
            theme(plot.margin=margin(t=5, r=0, b=5, l=5, unit="mm"))

        lon = c()
        lat = c()
        fill = c()
        
        for (code in Code) {
            
            df_data = list_df2plot[[i]]$data
            df_trend = list_df2plot[[i]]$trend
            p_threshold = list_df2plot[[i]]$p_threshold
            
            df_data_code = df_data[df_data$code == code,] 
            df_trend_code = df_trend[df_trend$code == code,]
            
            Start = Start_code[Code_code == code][[1]][idPer]
            End = End_code[Code_code == code][[1]][idPer]
            
            df_data_code_per =
                df_data_code[df_data_code$Date >= Start 
                             & df_data_code$Date <= End,]
            
            df_trend_code_per = 
                df_trend_code[df_trend_code$period_start == Start 
                              & df_trend_code$period_end == End,]
            
            Ntrend = nrow(df_trend_code_per)
            if (Ntrend > 1) {
                df_trend_code_per = df_trend_code_per[1,]
            }
            
            dataMean = mean(df_data_code_per$Qm3s, na.rm=TRUE)
            trendMean = df_trend_code_per$trend / dataMean
            
            if (df_trend_code_per$p <= p_threshold){
                color_res = get_color(trendMean, 
                                      minTrendMean[idPer, i],
                                      maxTrendMean[idPer, i],
                                      palette_name='perso',
                                      reverse=TRUE,
                                      ncolor=ncolor,
                                      nbTick=nbTick)
                
                filltmp = color_res$color
                palette = color_res$palette
            } else { 
                filltmp = 'grey97'
            }

            lontmp =
                df_meta[df_meta$code == code,]$L93X_m_BH            
            lattmp =
                df_meta[df_meta$code == code,]$L93Y_m_BH 

            lon = c(lon, lontmp)
            lat = c(lat, lattmp)
            fill = c(fill, filltmp)

            plot_map = tibble(lon=lon, lat=lat, fill=fill)

        }

        for (k in nCode) {
            map = map +
                geom_point(data=plot_map,
                           aes(x=lon, y=lat),
                           shape=21, size=5, stroke=1,
                           color='grey50', fill=fill)
        }


        idTick = color_res$idTick
        labTick = color_res$labTick
        colTick = color_res$colTick
        ncolorShow = length(color_res$palette)

        labTick = as.character(round(labTick*100, 2))
        
        xtick = rep(0, times=nbTick)

        plot_palette = tibble(xtick=xtick, ytick=idTick,
                              colTick=colTick, labTick=labTick)

        
        title = ggplot() + theme_void() +
            
            annotate('text',
                     x=-0.3, y=0.15,
                     label=bquote(bold(.(type))),
                     hjust=0, vjust=0,
                     size=10, color="#00A3A8") +
            
            geom_line(aes(x=c(-0.3, 3), y=c(0.05, 0.05)),
                      size=0.5, color="#00A3A8") +
            
            scale_x_continuous(limits=c(-1, 1 + 3),
                               expand=c(0, 0)) +
            
            scale_y_continuous(limits=c(0, 10),
                               expand=c(0, 0)) +
            
            theme(plot.margin=margin(t=5, r=5, b=0, l=0, unit="mm"))
        
        
        pal = ggplot() + theme_void() +
            
            geom_point(data=plot_palette,
                       aes(x=xtick, y=ytick),
                       shape=21, size=5, stroke=1,
                       color='grey50', fill=colTick)

        for (j in 1:nbTick) {
            pal = pal +
                annotate('text', x=xtick[j]+0.3,
                         y=idTick[j],
                         label=bquote(bold(.(labTick[j]))),
                         hjust=0, vjust=0.7, 
                         size=3, color='grey40')
        }

        pal = pal +
            
            geom_point(aes(x=0, y=-20),
                       shape=21, size=5, stroke=1,
                       color='grey50', fill='grey97') +
            
            annotate('text',
                     x=0.3, y=-20,
                     label=bquote(bold("non significatif  10%")),
                     hjust=0, vjust=0.7,
                     size=3, color='grey40')

        pal = pal +
            
            annotate('text',
                     x=-0.3, y=ncolorShow + 29,
                     label="Tendance",
                     hjust=0, vjust=0.5,
                     size=6, color='grey40') +
            
            annotate('text',
                     x=-0.2, y=ncolorShow + 16,
                     label=bquote(bold("% par an")),
                     hjust=0, vjust=0.5,
                     size=4, color='grey40')
            
        pal = pal +
            
            scale_x_continuous(limits=c(-1, 1 + 3),
                               expand=c(0, 0)) +
            
            scale_y_continuous(limits=c(-55, ncolorShow + 45),
                               expand=c(0, 0)) +
            
            theme(plot.margin=margin(t=0, r=5, b=5, l=0, unit="mm"))
        

        Map = list(map, title, pal)
        
        plot = grid.arrange(grobs=Map, layout_matrix=
                                           matrix(c(1, 1, 1, 2,
                                                    1, 1, 1, 3),
                                                  nrow=2, byrow=TRUE))
        
        # Saving matrix plot
        ggsave(plot=plot,
               path=outdirTmp,
               filename=paste(outname, '.pdf', sep=''),
               width=29.7, height=21, units='cm', dpi=100)
    }
}




text_panel = function(code, df_meta) {
    df_meta_code = df_meta[df_meta$code == code,]

    text1 = paste(
        "<b>", code, '</b>  -  ', df_meta_code$nom, ' &#40;',
        df_meta_code$region_hydro, '&#41;', 
        sep='')

    text2 = paste(
        "<b>",
        "Gestionnaire : ", df_meta_code$gestionnaire, "<br>", 
        "</b>",
        sep='')

    text3 = paste(
        "<b>",
        "Superficie : ", df_meta_code$surface_km2_BH, "  [km<sup>2</sup>] <br>",
        "X = ", df_meta_code$L93X_m_BH, "  [m ; Lambert 93]", 
        "</b>",
        sep='')
        
    text4 = paste(
        "<b>",
        "Altitude : ", df_meta_code$altitude_m_BH, "  [m]<br>",
        "Y = ", df_meta_code$L93Y_m_BH, "  [m ; Lambert 93]",
        "</b>",
        sep='')

    # text3 = paste(
    #     "<b>",
    #     "Superficie : ", df_meta_code$surface_km2_IN, 
    #     ' (', df_meta_code$surface_km2_BH, ')', "  [km<sup>2</sup>] <br>",
    #     "X = ", df_meta_code$L93X_m_IN, 
    #     ' (', df_meta_code$L93X_m_BH, ')', "  [m ; Lambert 93]", 
    #     "</b>",
    #     sep='')
        
    # text4 = paste(
    #     "<b>",
    #     "Altitude : ", df_meta_code$altitude_m_IN, 
    #     ' (', df_meta_code$altitude_m_BH, ')', "  [m]<br>",
    #     "Y = ", df_meta_code$L93Y_m_IN, 
    #     ' (', df_meta_code$L93Y_m_BH, ')', "  [m ; Lambert 93]",
    #     "</b>",
    #     sep='')

    # text5 = paste(
    #     "<b>",
    #     "INRAE (Banque Hydro)<br>",
    #     "INRAE (Banque Hydro)",
    #     "</b>",
    #     sep='')

    gtext1 = richtext_grob(text1,
                           x=0, y=1,
                           margin=unit(c(t=5, r=5, b=0, l=5), "mm"),
                           hjust=0, vjust=1,
                           gp=gpar(col="#00A3A8", fontsize=14))

    gtext2 = richtext_grob(text2,
                           x=0, y=0.55,
                           margin=unit(c(t=0, r=5, b=0, l=5), "mm"),
                           hjust=0, vjust=1,
                           gp=gpar(col="grey20", fontsize=8))
    
    gtext3 = richtext_grob(text3,
                           x=0, y=1,
                           margin=unit(c(t=0, r=5, b=5, l=5), "mm"),
                           hjust=0, vjust=1,
                           gp=gpar(col="grey20", fontsize=9))
    
    gtext4 = richtext_grob(text4,
                           x=0, y=1,
                           margin=unit(c(t=0, r=5, b=5, l=5), "mm"),
                           hjust=0, vjust=1,
                           gp=gpar(col="grey20", fontsize=9))

    # gtext5 = richtext_grob(text5,
                           # x=0, y=1,
                           # margin=unit(c(t=0, r=5, b=5, l=5), "mm"),
                           # hjust=0, vjust=1,
                           # gp=gpar(col="grey20", fontsize=9))
    
    gtext_merge = grid.arrange(grobs=list(gtext1, gtext2, gtext3, 
                                          gtext4),#, gtext5), 
                               layout_matrix=matrix(c(1, 1, 1,
                                                      2, 2, 2,
                                                      3, 4, 5),
                                                    nrow=3, 
                                                    byrow=TRUE))
    return(gtext_merge)
}


histogram = function (data_bin, df_meta, figdir='', filedir_opt='') {

    # Get all different stations code
    Code = levels(factor(df_meta$code))
    nCode = length(Code)

    # If there is not a dedicated figure directory it creats one
    outdir = file.path(figdir, filedir_opt, sep='')
    if (!(file.exists(outdir))) {
        dir.create(outdir)
    }

    datebreak = 10
    dateminbreak = 1

    res_hist = hist(data_bin, breaks='years', plot=FALSE)
    counts = res_hist$counts
    counts_pct = counts/nCode * 100
    breaks = as.Date(res_hist$breaks)
    mids = as.Date(res_hist$mids)
    
    p = ggplot() + theme_ash +

    theme(panel.grid.major.y=element_line(color='grey85', size=0.15),
          axis.title.y=element_blank()) +

        geom_bar(aes(x=mids, y=counts_pct), 
                 stat='identity',
                 fill="#00A3A8") +

        scale_x_date(date_breaks=paste(as.character(datebreak), 
                                       'year', sep=' '),
                     date_minor_breaks=paste(as.character(dateminbreak), 
                                             'year', sep=' '),
                     guide='axis_minor',
                     date_labels="%Y",
                     limits=c(min(data_bin)-years(0), 
                              max(data_bin)+years(0)),
                     expand=c(0, 0)) +

        scale_y_continuous(limits=c(0,
                                    max(counts_pct)*1.1),
                           expand=c(0, 0))
    
    ggsave(plot=p, 
           path=outdir,
           filename=paste('hist_break_date', '.pdf', sep=''),
           width=10, height=10, units='cm', dpi=100)
}


cumulative = function (data_bin, df_meta, dyear=10, figdir='', filedir_opt='') {

    # Get all different stations code
    Code = levels(factor(df_meta$code))
    nCode = length(Code)

    # If there is not a dedicated figure directory it creats one
    outdir = file.path(figdir, filedir_opt, sep='')
    if (!(file.exists(outdir))) {
        dir.create(outdir)
    }

    datebreak = 10
    dateminbreak = 1

    res_hist = hist(data_bin, breaks='years', plot=FALSE)
    counts = res_hist$counts
    cumul = cumsum(counts)
    cumul_pct = cumul/nCode * 100
    breaks = as.Date(res_hist$breaks)
    mids = as.Date(res_hist$mids)

    mids = c(mids[1] - years(dyear), mids[1] - years(1),
               mids,
               mids[length(mids)] + years(dyear)) 
    cumul_pct = c(0, 0, 
                  cumul_pct,
                  cumul_pct[length(cumul_pct)])

    mids = mids + months(6)

    breaks = breaks + 1
    breaks = breaks[-length(breaks)]

    DB = c()
    for (i in 1:length(breaks)) {
        DB = c(DB, rep(breaks[i], times=counts[i]))
    }
    q50 = as.Date(quantile(DB, probs=0.5)) + years(1)
    
    print(paste('mediane :', q50))
    
    p = ggplot() + theme_ash +

    theme(panel.grid.major.y=element_line(color='grey85', size=0.15),
          axis.title.y=element_blank()) +

        geom_line(aes(x=mids, y=cumul_pct), 
                  color="#00A3A8") +
    
        geom_line(aes(x=c(q50, q50), y=c(0, 100)), 
                  color="wheat", 
                  lty='dashed') +

        scale_x_date(date_breaks=paste(as.character(datebreak), 
                                       'year', sep=' '),
                     date_minor_breaks=paste(as.character(dateminbreak), 
                                             'year', sep=' '),
                     guide='axis_minor',
                     date_labels="%Y",
                     limits=c(min(mids)-years(0), 
                              max(mids)+years(0)),
                     expand=c(0, 0)) +

        scale_y_continuous(limits=c(-1, 101),
                           expand=c(0, 0))
    
    ggsave(plot=p, 
           path=outdir,
           filename=paste('cumul_break_date', '.pdf', sep=''),
           width=10, height=10, units='cm', dpi=100)
}


get_color = function (value, min, max, ncolor=256, palette_name='perso', reverse=FALSE, nbTick=10) {
    
    if (palette_name == 'perso') {
        palette = colorRampPalette(c(
            # '#1a4157',
            # '#00af9d',
            # '#fbdd7e',
            # '#fdb147',
            # '#fd4659'

            '#0f3b57',
            '#1d7881',
            '#80c4a9',
            '#e2dac6', #mid
            '#fadfad',
            '#d08363',
            '#7e392f'

            # '#193830',
            # '#2A6863',
            # '#449C93',
            # '#7ACEB9',
            # '#BCE6DB',
            # '#EFE0B0',
            # '#D4B86A',
            # '#B3762A',
            # '#7F4A23',
            # '#452C1A'

        ))(ncolor)
        
    } else {
        palette = colorRampPalette(brewer.pal(11, palette_name))(ncolor)
    }

    if (reverse) {
        palette = rev(palette)
    }
    
    palette_cold = palette[1:as.integer(ncolor/2)]
    palette_hot = palette[(as.integer(ncolor/2)+1):ncolor]

    ncolor_cold = length(palette_cold)
    ncolor_hot = length(palette_hot)

    if (value < 0) {
        idNorm = (value - min) / (0 - min)
        id = round(idNorm*(ncolor_cold - 1) + 1, 0)
        color = palette_cold[id]
    } else {
        idNorm = (value - 0) / (max - 0)
        id = round(idNorm*(ncolor_hot - 1) + 1, 0)
        color = palette_hot[id]
    }    

    if (min < 0 & max < 0) {
        paletteShow = palette_cold
    } else if (min > 0 & max > 0) {
        paletteShow = palette_hot
    } else {
        paletteShow = palette
    }

    ncolorShow = length(paletteShow)
    
    idTick = c()
    for (i in 1:nbTick) {
        id = round((ncolorShow-1)/(nbTick-1)*(i-1)) + 1
        idTick = c(idTick, id)
    }
    
    labTick = seq(min, max, length.out=nbTick)
    colTick = paletteShow[idTick]
    
    return(list(color=color, palette=paletteShow,
                idTick=idTick, labTick=labTick, colTick=colTick))
}


void = ggplot() + geom_blank(aes(1,1)) +
    theme(
        plot.background = element_blank(), 
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(), 
        panel.border = element_blank(),
        panel.background = element_blank(),
        axis.title.x = element_blank(),
        axis.title.y = element_blank(),
        axis.text.x = element_blank(), 
        axis.text.y = element_blank(),
        axis.ticks = element_blank(),
        axis.line = element_blank()
    )


palette_tester = function (n=256) {

    X = 1:n
    Y = rep(0, times=n)

    palette = colorRampPalette(c(
        # '#1a4157',
        # '#00af9d',
        # '#fbdd7e',
        # '#fdb147',
        # '#fd4659'
        
        # '#543005',
        # '#8c510a',
        # '#bf812d',
        # '#dfc27d',
        # '#f6e8c3',
        # '#f5f5f5',
        # '#c7eae5',
        # '#80cdc1',
        # '#35978f',
        # '#01665e',
        # '#003c30'
        
        '#0f3b57',
        '#1d7881',
        '#80c4a9',
        '#e2dac6', #mid
        '#fadfad',
        '#d08363',
        '#7e392f'

        # '#193830',
        # '#2A6863',
        # '#449C93',
        # '#7ACEB9',
        # '#BCE6DB',
        # '#EFE0B0',
        # '#D4B86A',
        # '#B3762A',
        # '#7F4A23',
        # '#452C1A'

    ))(n)

    p = ggplot() + 

    theme(
        plot.background = element_blank(), 
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(), 
        panel.border = element_blank(),
        panel.background = element_blank(),
        axis.title.x = element_blank(),
        axis.title.y = element_blank(),
        axis.text.x = element_blank(), 
        axis.text.y = element_blank(),
        axis.ticks = element_blank(),
        axis.line = element_blank()
    ) +
    
        geom_line(aes(x=X, y=Y), color=palette[X], size=60) +
        scale_y_continuous(expand=c(0, 0))

    ggsave(plot=p,
           filename=paste('palette_test', '.pdf', sep=''),
           width=10, height=10, units='cm', dpi=100)
}

# palette_tester()


get_power = function (value) {
    
    if (value > 1) {
        power = nchar(as.character(as.integer(value))) - 1
    } else {
        dec = gsub('0.', '', as.character(value), fixed=TRUE)
        ndec = nchar(dec)
        nnum = nchar(as.character(as.numeric(dec)))
        power = -(ndec - nnum + 1)
    }
    
    return(power)
}


gg_circle = function(r, xc, yc, color="black", fill=NA, ...) {
    x = xc + r*cos(seq(0, pi, length.out=100))
    ymax = yc + r*sin(seq(0, pi, length.out=100))
    ymin = yc + r*sin(seq(0, -pi, length.out=100))
    annotate("ribbon", x=x, ymin=ymin, ymax=ymax, color=color, fill=fill, ...)
}



gpct = function (pct, L, ref=NULL, shift=FALSE) {
    
    if (is.null(ref)) {
        minL = min(L, na.rm=TRUE)
    } else {
        minL = ref
    }
    
    maxL = max(L, na.rm=TRUE)
    spanL = maxL - minL
 
    xL = pct/100 * as.numeric(spanL)

    if (shift) {
        xL = xL + minL
    }
    return (xL)
}
